package com.alivc.live.pusher.demo;

import static android.os.Environment.MEDIA_MOUNTED;

import android.app.Application;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.net.ConnectivityManager;
import android.os.Environment;
import android.util.Log;

import com.alivc.live.pusher.AlivcLiveBase;
import com.alivc.live.pusher.AlivcLivePushInstance;
import com.alivc.live.pusher.AlivcLivePushLogLevel;
import com.alivc.live.pusher.demo.test.PushDemoTestConstants;
import com.aliyun.animoji.AnimojiDataFactory;
import com.aliyun.animoji.utils.LogUtils;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;

public class LiveApplication extends Application {
    private static final String TAG = LiveApplication.class.getSimpleName();

    @Override
    protected void attachBaseContext(Context base) {
        super.attachBaseContext(base);
    }

    @Override
    public void onCreate() {
        super.onCreate();
        ContextUtils.setContext(this);
        IntentFilter filter = new IntentFilter();
        filter.addAction(ConnectivityManager.CONNECTIVITY_ACTION);
        registerReceiver(new ConnectivityChangedReceiver(), filter);
        AlivcLivePushInstance.loadInstance(this);

        // 日志配置
        AlivcLiveBase.setLogLevel(AlivcLivePushLogLevel.AlivcLivePushLogLevelDebug);
        AlivcLiveBase.setConsoleEnabled(true);
        String logPath = getFilePath(getApplicationContext(), "log_path");
        // full log file limited was kLogMaxFileSizeInKB * 5 (parts)
        int maxPartFileSizeInKB = 100 * 1024 * 1024; //100G
        AlivcLiveBase.setLogDirPath(logPath, maxPartFileSizeInKB);

        AnimojiDataFactory.INSTANCE.loadResources(this);
        LogUtils.init(true);
        PushDemoTestConstants.printVersionInfo();
    }

    public static String getFilePath(Context context, String dir) {
        String logFilePath = "";
        //判断SD卡是否可用
        if (MEDIA_MOUNTED.equals(Environment.getExternalStorageState())) {
            logFilePath = context.getExternalFilesDir(dir).getAbsolutePath();
        } else {
            //没内存卡就存机身内存
            logFilePath = context.getFilesDir() + File.separator + dir;
        }
        File file = new File(logFilePath);
        if (!file.exists()) {//判断文件目录是否存在
            file.mkdirs();
        }

        //Set log folder path in 4.4.0+ version
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String logFileName = "live_pusher_" + sdf.format(new Date()) + "_" + String.valueOf(System.currentTimeMillis()) + ".log";
        logFilePath += File.separator + logFileName;

        Log.d(TAG, "log filePath====>" + logFilePath);
        return logFilePath;
    }

    class ConnectivityChangedReceiver extends BroadcastReceiver {
        @Override
        public void onReceive(Context context, Intent intent) {

        }
    }
}
